<?php
/**
 * @package Log
 */
ob_start();

require_once 'FirePHPCore/FirePHP.class.php';

/**
 * The Log_firephp class is a concrete implementation of the Log::
 * abstract class which writes message into FirePHP console.
 *
 * http://www.firephp.org/
 *
 * @author  Laurent Laville <pear@laurent-laville.org>
 * @since   Log 1.12.0
 * @package Log
 *
 * @example firephp.php     Using the firephp handler.
 */
class Log_firephp extends Log
{
    /**
     * String containing the format of a log line.
     * @var string
     * @access private
     */
    var $_lineFormat = '%2$s [%3$s] %4$s';

    /**
     * String containing the timestamp format.  It will be passed directly to
     * strftime().  Note that the timestamp string will generated using the
     * current locale.
     *
     * Note! Default lineFormat of this driver does not display time.
     *
     * @var string
     * @access private
     */
    var $_timeFormat = '%b %d %H:%M:%S';

    /**
     * Constructs a new Log_firephp object.
     *
     * @param string $name     Ignored.
     * @param string $ident    The identity string.
     * @param array  $conf     The configuration array.
     * @param int    $level    Log messages up to and including this level.
     * @access public
     */
    function Log_firephp($name = '', $ident = 'PHP', $conf = array(),
                         $level = PEAR_LOG_DEBUG)
    {
        $this->_id = md5(microtime());
        $this->_ident = $ident;
        $this->_mask = Log::UPTO($level);

        if (!empty($conf['lineFormat'])) {
            $this->_lineFormat = str_replace(array_keys($this->_formatMap),
                                             array_values($this->_formatMap),
                                             $conf['lineFormat']);
        }

        if (!empty($conf['timeFormat'])) {
            $this->_timeFormat = $conf['timeFormat'];
        }
    }

    /**
     * Writes $message to firephp console. Also, passes the message
     * along to any Log_observer instances that are observing this Log.
     *
     * @param mixed  $message    String or object containing the message to log.
     * @param string $priority The priority of the message.  Valid
     *                  values are: PEAR_LOG_EMERG, PEAR_LOG_ALERT,
     *                  PEAR_LOG_CRIT, PEAR_LOG_ERR, PEAR_LOG_WARNING,
     *                  PEAR_LOG_NOTICE, PEAR_LOG_INFO, and PEAR_LOG_DEBUG.
     * @return boolean  True on success or false on failure.
     * @access public
     */
    function log($message, $priority = null)
    {
        /* If a priority hasn't been specified, use the default value. */
        if ($priority === null) {
            $priority = $this->_priority;
        }

        /* Abort early if the priority is above the maximum logging level. */
        if (!$this->_isMasked($priority)) {
            return false;
        }

        $is_exception = false;
        // to allow PHP4 users to use this driver too
        if (version_compare(phpversion(), '5.0.0', '>=')) {
            if ($message instanceof Exception) {
                $is_exception = true;
            }
        }

        if (is_array($message)) {
            $arg = func_num_args() - 1;
            if (func_get_arg($arg) == FirePHP::TABLE) {
                // table
            } else {
                // dump
                $line = $message;
            }
        } elseif ($is_exception === true) {
            $args = array($message, FirePHP::EXCEPTION);
        } elseif (is_string($message)
                  && func_num_args() == 3 && func_get_arg(2) == FirePHP::TRACE) {
            $line = $this->_extractMessage($message);
        } else {
            /* Extract the string representation of the message. */
            $message = $this->_extractMessage($message);

            /* normalize line breaks */
            $message = str_replace("\r\n", "\n", $message);

            /* escape line breaks */
            $message = str_replace("\n", "\\n\\\n", $message);

            /* escape quotes */
            $message = str_replace('"', '\\"', $message);

            /* Build the string containing the complete log line. */
            $line = $this->_format($this->_lineFormat,
                                   strftime($this->_timeFormat),
                                   $priority,
                                   $message);

            switch ($priority) {
            case PEAR_LOG_EMERG:
            case PEAR_LOG_ALERT:
            case PEAR_LOG_CRIT:
            case PEAR_LOG_ERR:
                $args = array($line, FirePHP::ERROR);
                break;
            case PEAR_LOG_WARNING:
                $args = array($line, FirePHP::WARN);
                break;
            case PEAR_LOG_INFO:
                $args = array($line, FirePHP::INFO);
                break;
            case PEAR_LOG_NOTICE:
            case PEAR_LOG_DEBUG:
            default :
                $args = array($line, FirePHP::LOG);
                break;
            }
        }
        $instance = FirePHP::getInstance(true);

        if (func_num_args() == 1) {
            // fb($Object);
        } elseif (func_num_args() == 2) {
            // fb($Object, $Type);
        } elseif (func_num_args() == 3) {
            if (func_get_arg(2) == FirePHP::TRACE
                || func_get_arg(2) == FirePHP::DUMP) {
                // fb($Object, FirePHP::TRACE);
                // fb($Object, FirePHP::DUMP);
                $args = array($line, func_get_arg(2));
            } elseif (func_get_arg(2) == FirePHP::TABLE) {
                // fb($Object, FirePHP::TABLE);
                $args = array($message, func_get_arg(2));
            } else {
                // fb($Object, $Label, $Type);
                $args = array($line, func_get_arg(2), array_pop($args));
            }
        } elseif (func_num_args() == 4) {
            // fb($Object, $Label, FirePHP::DUMP);
            $args = array($line, func_get_arg(2), func_get_arg(3));
        }
        call_user_func_array(array($instance,'fb'), $args);

        /* Notify observers about this log message. */
        $this->_announce(array('priority' => $priority, 'message' => $message));

        return true;
    }
}
?>